from ._malee import *


__version__ = "0.0.3"
__all__ = ["ar", "lr", "aar", "twr", "an", "pv", "npv", "gm"]
__docformat__ = "google"

# module level doc-string
__doc__ = """
mali - a simple financial modeling library for Python
=====================================================================
**mali** is a Python package providing simple, easy to undestand, and useful
library designed to make people lives a little easier when dealing with financial
modeling using Python. At the moment it aims to be easy enough so people feel home when 
using it. Additionally, it has the broader goal of becoming **the goto library when people
consider doing financial modeling in Python**. It is already well on its way toward this 
goal.

**mali** in Farsi means Financial. Tried so many other cool names, but I was late in the game
and they were already taken.

Main Features
-------------
Here are just a few of the things that ``mali`` has at the moment:
  - A list of necessary rate of returns and risk functions ready to be used 
  - Easy to undestand code base so anyone can contribute to.
  - Enough documentation to get you started.
  - Full test coverage so everything is checked before adding/changing functionality
"""


"""`malee` is a collection of essential financial modeling 
functions to make your life easier when dealing with finance 
in python. 
"""
from __future__ import division, absolute_import, print_function
from math import log, e


def ar(vi, vf, D=0, W=0):
    """The Arithmetic Return is the simplest way of calculating
    the rate of return on an investment. To calculate it,
    you need the amount of growth, which is simply the final value `Vf`
    minus the initial value `Vi`. Then you just divide the amount of growth
    by the initial amount.

    Args:
        vi: Initial value of investment
        vf: Final value of investment
        D: The total deposit made into the investment
        W: Total of any withdrawals

    Returns:
        The arithmetic return of a given investment.

    Example:
        By providing initial and final value of investment
        you can get the percentage return of your investment:

        >>> import malee
        >>> malee.ar(100, 140)
        0.4
    """
    return (vf - D + W - vi) / vi


def lr(vi, vf, t=1):
    """The logarithmic return is a way of calculating
    the rate of return on an investment. To calculate it
    you need the inital value of the investment `Vi`,
    the final value `Vf` and the number of time periods `t`.
    You then take the natural logarithm of `Vf` divided by `Vi`,
    and divide the result by `t`.

    Args:
        vi: Initial value of investment
        vf: Final value of investment
        t: Number of time periods

    Returns:
        The logarithmic return of a given investment

    Example:
        By providing initial and final value of investment
        you can get the percentage of logarithmic return of
        your investment:

        >>> import malee
        >>> malee.lr(100, 140)
        0.3364722366212129
    """
    return log(vf / vi, e) / t


def aar(rors):
    """The Arithmetic Average Return is a way of calculating
    an average return for an investment over multiple periods.
    It is simply the average of all the arithmetic returns for
    each period. To calculate it, you add up the individual
    Arithmetic Return values, rarith, for each period, then
    divide by the number of periods `n`.

    Args:
        rors: List of all rate of returns over multiple time periods

    Returns:
        The arithmetic average return of a given investment

    Example:
        Here's how you can calculate the arithmetic average
        return by providing a list of individual returns in
        multiple periods of time:

        >>> import malee
        >>> malee.aar([.1, -.05, .07])
        0.04
    """
    R = 0
    for ror in rors:
        R += ror
    R /= len(rors)
    return R


def twr(rors):
    """The Time-Weighted Return (also called the
    Geometric Average Return) is a way of calculating
    the rate of return for an investment when there are
    deposits and withdrawals (cash flows) during the period.
    You often want to exclude these cash flows so that we can
    find out how well the underlying investment has performed.

    Args:
        rors: List of all rors over multiple time periods

    Returns:
        The time-weighted return of a given investment

    Example:
        By providing a list of rate of returns you can
        calculate the time-weighted return based on that
        list:

        >>> import malee
        >>> malee.aar([.10, -.05, .12])
        0.056666666666666664
    """
    Rtw = 1
    for ror in rors:
        Rtw *= 1 + ror
    return Rtw - 1


def an(Rtw, y=1):
    """The time-weighted return expressed as an annual rate,
    then you need to annualize it using this function.

    Args:
        Rtw: Time-weighted return
        y: Number of years for the period

    Returns:
        Time-weighted return expressed as an annual rate

    Example:
        If you want to express your time-weighted return
        as an annual rate, here's how you can do it for 2
        years period:

        >>> import malee
        >>> malee.an(0.3662, 2)
        0.1688455843266894
    """
    return (1 + Rtw) ** (1 / y) - 1


def pv(Rt, i, t=1):
    """Present value accounts for the current value of a future
    sum of money or stream of cash flows given a specified
    rate of return.

    Args:
        Rt: Return we get at time `t`
        i: Discount rate
        t: Time we get the return

    Returns:
        Present value

    Example:
        Here's a simple example of how you can calculate
        the present value of a given return and time period:

        >>> import malee
        >>> malee.pv(1030, 0.05, 1)
        980.952380952381
    """
    return Rt / ((1 + i) ** t)


def npv(Rn, i, i0, pe=0):
    """Net present value (NPV) is the difference between
    the present value of cash inflows and the present value
    of cash outflows over a period of time.

    Args:
        Rn: Expected return list
        i: Discount rate
        i0: Initial amount invested
        pe: Profit or expense at the end of investment

    Returns:
        Net present value

    Example:
        Given the expected return list `Rn`, `i` as
        discount rate, and `i0` as initial amount invested
        you can calcuate NPV like this:

        >>> import malee
        >>> malee.npv([5000, 8000, 12000, 30000], 0.05, 40000)
        7065.266015703324
    """
    npv_sum = 0
    for idx, Ri in enumerate(Rn):
        if Ri == Rn[-1] and pe != 0:
            npv_sum += Ri + pe / ((1 + i) ** (idx + 1))
        else:
            npv_sum += Ri / ((1 + i) ** (idx + 1))
    return npv_sum - i0


def gm(rors):
    """The Geometric Mean is the average of a set of products,
    the calculation of which is commonly used to determine the
    performance results of an investment or portfolio. It is
    technically defined as "the nth root product of n numbers."

    Args:
        rors: List of all rors over multiple time periods

    Returns:
        The Geometric Mean of a list of returns

    Example:
        By providing a list of returns you can calculate
        the Geometric Mean of the given list:

        >>> import malee
        >>> malee.gm([.14, .06, -.05, .20])
        0.08337520558323552
    """
    total = 1
    n = len(rors)
    for ror in rors:
        total *= 1 + ror
    return total ** (1 / n) - 1



#!/usr/bin/env python

from os import path


def get_version():
    """
    Find the value assigned to __version__ in malee/__init__.py.
    This function assumes that there is a line of the form
        __version__ = "version-string"
    in that file.  It returns the string version-string, or None if such a
    line is not found.
    """
    with open("malee/__init__.py", "r") as f:
        for line in f:
            s = [w.strip() for w in line.split("=", 1)]
            if len(s) == 2 and s[0] == "__version__":
                return s[1][1:-1]


CLASSIFIERS = """\
Development Status :: 5 - Production/Stable
Intended Audience :: Education
Intended Audience :: Financial and Insurance Industry
License :: OSI Approved :: BSD License
Programming Language :: Python
Programming Language :: Python :: 3
Programming Language :: Python :: 3.5
Programming Language :: Python :: 3.6
Programming Language :: Python :: 3.7
Programming Language :: Python :: 3 :: Only
Topic :: Software Development
Topic :: Office/Business :: Financial :: Accounting
Topic :: Office/Business :: Financial :: Investment
Topic :: Office/Business :: Financial :: Spreadsheet
"""

here = path.abspath(path.dirname(__file__))
with open(path.join(here, "README.md"), "r") as f:
    long_description = f.read()

if __name__ == "__main__":
    from setuptools import setup, find_packages

    setup(
        name="malee",
        version=get_version(),
        description="Malee",
        long_description=long_description,
        long_description_content_type="text/markdown",
        packages=["malee"],
        install_requires=[],
        author="Reza Behzadpour",
        license="BSD 3-Clause",
        maintainer="Reza Behzadpour",
        maintainer_email="reza.behzadpour1993@gmail.com",
        python_requires=">=3.5",
        classifiers=CLASSIFIERS.splitlines(),
        url="https://malee.readthedocs.io",
        download_url="https://pypi.org/project/malee/",
        project_urls={
            "Bug Tracker": "https://github.com/RezaBehzadpour/malee/issues",
            "Documentation": "https://malee.readthedocs.io",
            "Source Code": "https://github.com/RezaBehzadpour/malee",
        },
    )
